<?php

/**
 * @Contains \Drupal\search_api_page\Controller\SearchApiPageController.
 */

namespace Drupal\search_api_page\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\search_api\Entity\Index;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Item\ItemInterface;
use Drupal\search_api_page\Entity\SearchApiPage;
use Drupal\search_api_page\SearchApiPageInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Defines a controller to serve search pages.
 */
class SearchApiPageController extends ControllerBase {

  public function page(Request $request, $search_api_page, $keys = '') {
    $build = array();

    /** @var $searchApiPage SearchApiPageInterface */
    $searchApiPage = SearchApiPage::load($search_api_page);

    // Keys can be in the query.
    if (!$searchApiPage->getCleanUrl()) {
      $keys = $request->get('keys');
    }

    // Page title.
    $build['#title'] = $searchApiPage->label();

    // Show the search form.
    if ($searchApiPage->showSearchForm()) {
      $args = array(
        'search_api_page' => $searchApiPage->id(),
        'keys' => $keys,
      );
      $build['#form'] = $this->formBuilder()->getForm('Drupal\search_api_page\Form\SearchApiPageBlockForm', $args);
    }

    $perform_search = TRUE;
    if (empty($keys) && !$searchApiPage->showAllResultsWhenNoSearchIsPerformed()) {
      $perform_search = FALSE;
    }

    if ($perform_search) {

      /** @var $searchApiIndex IndexInterface */
      $searchApiIndex = Index::load($searchApiPage->getIndex());

      // Create the query.
      $query = $searchApiIndex->query([
        'parse_mode' => 'direct',
        'limit' => $searchApiPage->getLimit(),
        'offset' => !is_null($request->get('page')) ? $request->get('page') : 0,
        'search id' => 'search_api_page:' . $searchApiPage->id()
      ]);

      // Search for keys.
      if (!empty($keys)) {
        $query->keys($keys);
      }

      // Index fields.
      $query->setFulltextFields($searchApiPage->getSearchedFields());

      $result = $query->execute();
      $items = $result->getResultItems();

      /** @var $item ItemInterface*/
      $results = array();
      foreach ($items as $item) {

        list(, $path, $langcode) = explode(':', $item->getId());
        list($entity_type, $id) = explode('/', $path);

        // Get the entity.
        $entity = $this->entityTypeManager()->getStorage($entity_type)->load($id);

        // Render as view modes.
        if ($searchApiPage->renderAsViewModes()) {
          // TODO allow to configure view mode.
          $results[] = $this->entityTypeManager()->getViewBuilder($entity_type)->view($entity, 'teaser');
        }

        // Render as snippets.
        if ($searchApiPage->renderAsSnippets()) {
          $results[] = array(
            '#theme' => 'search_api_page_result',
            '#item' => $item,
            '#entity' => $entity,
          );
        }
      }


      if (!empty($results)) {

        $build['#search_title'] = array(
          '#markup' => $this->t('Search results'),
        );

        $build['#no_of_results'] = array(
          '#markup' => $this->formatPlural($result->getResultCount(), '1 result found', '@count results found'),
        );

        $build['#results'] = $results;

        // Build pager.
        pager_default_initialize($result->getResultCount(), $searchApiPage->getLimit());
        $build['#pager'] = array(
          '#type' => 'pager',
        );
      }
      elseif ($perform_search) {
        $build['#no_results_found'] = array(
          '#markup' => $this->t('Your search yielded no results.'),
        );

        $build['#search_help'] = array(
          '#markup' => $this->t('<ul>
<li>Check if your spelling is correct.</li>
<li>Remove quotes around phrases to search for each word individually. <em>bike shed</em> will often show more results than <em>&quot;bike shed&quot;</em>.</li>
<li>Consider loosening your query with <em>OR</em>. <em>bike OR shed</em> will often show more results than <em>bike shed</em>.</li>
</ul>'),
        );
      }
    }

    $build['#theme'] = 'search_api_page';

    // TODO caching dependencies

    return $build;
  }

}
